#!/bin/sh

# Author: Gemma Ragozzine
# Thank you to Stephen Edward's MicroC example and Lorax Example from Fall 2013
#

graphquil="./graphquil"
java_output="java graphQuil"
java_file="graphQuil"

# Set time limit for all operations
ulimit -t 30

globallog=testall.log
rm -f $globallog
error=0
globalerror=0

keep=0
chmod +x java

Usage() {
    echo "Usage: testall.sh [options] [.gq files]"
    echo "-k    Keep intermediate files"
    echo "-h    Print this help"
    exit 1
}

SignalError() {
    if [ $error -eq 0 ] ; then
	echo "FAILED"
	error=1
    fi
    echo "  $1"
}

# Compare <outfile> <expected_file> <difffile>
# Compares the outfile with reffile.  Differences, if any, written to difffile
Compare() {
    generatedfiles="$generatedfiles $3"
    echo diff -b $1 $2 ">" $3 1>&2
    diff -b "$1" "$2" > "$3" 2>&1 || {
	SignalError "$1 differs"
	echo "FAILED $1 differs from $2" 1>&2
    }
}

# Run <args>
# Report the command, run it, and report any errors
Run() {
    echo $* 1>&2
    eval $* || {
	SignalError "$1 failed on $*"
	return 1
    }
}


CheckParser() {
    error=0
    basename=`echo $1 | sed 's/.*\\///
                             s/.gq//'`
    reffile=`echo $1 | sed 's/.gq$//'`
    basedir="`echo $1 | sed 's/\/[^\/]*$//'`/."

    echo -n "$basename..."

    echo 1>&2
    echo "###### Testing $basename" 1>&2

    generatedfiles=""

    generatedfiles="$generatedfiles ${basename}.a.out" &&
    Run "$graphquil" "-a" $1 ">" ${basename}.a.out &&
    Compare ${basename}.a.out ${reffile}.out ${basename}.a.diff

    if [ $error -eq 0 ] ; then
    if [ $keep -eq 0 ] ; then
        rm -f $generatedfiles
    fi
    echo "OK"
    echo "###### SUCCESS" 1>&2
    else
    echo "###### FAILED" 1>&2
    globalerror=$error
    fi 
}

CheckSemanticAnalysis() {
    error=0
    basename=`echo $1 | sed 's/.*\\///
                             s/.gq//'`
    reffile=`echo $1 | sed 's/.gq$//'`
    basedir="`echo $1 | sed 's/\/[^\/]*$//'`/."

    echo -n "$basename..."

    echo 1>&2
    echo "###### Testing $basename" 1>&2

    generatedfiles=""

    generatedfiles="$generatedfiles ${basename}.s.out" &&
    Run "$graphquil" "-s" $1 ">" ${basename}.s.out &&
    Compare ${basename}.s.out ${reffile}.out ${basename}.s.diff

    if [ $error -eq 0 ] ; then
    if [ $keep -eq 0 ] ; then
        rm -f $generatedfiles
    fi
    echo "OK"
    echo "###### SUCCESS" 1>&2
    else
    echo "###### FAILED" 1>&2
    globalerror=$error
    fi 
}

#Right now, tests failures up to semantic check point
CheckFail() {
    error=0
    basename=`echo $1 | sed 's/.*\\///
                             s/.gq//'`
    reffile=`echo $1 | sed 's/.gq$//'`
    basedir="`echo $1 | sed 's/\/[^\/]*$//'`/."

    echo -n "$basename..."

    echo 1>&2
    echo "###### Testing $basename" 1>&2

    generatedfiles=""

    # old from graphquil - interpreter
    # generatedfiles="$generatedfiles ${basename}.i.out" &&
    # Run "$graphquil" "-i" "<" $1 ">" ${basename}.i.out &&
    # Compare ${basename}.i.out ${reffile}.out ${basename}.i.diff

    generatedfiles="$generatedfiles ${basename}.s.out" && 
    {Run "$graphquil" "-s" $1 "2>" ${basename}.s.out} &&
    Compare ${basename}.s.out ${reffile}.out ${basename}.s.diff

    # Report the status and clean up the generated files

    if [ $error -eq 0 ] ; then
    if [ $keep -eq 0 ] ; then
        rm -f $generatedfiles
    fi
    echo "OK"
    echo "###### SUCCESS" 1>&2
    else
    echo "###### FAILED" 1>&2
    globalerror=$error
    fi
}

Check() {
    error=0
    basename=`echo $1 | sed 's/.*\\///
                             s/.gq//'`
    reffile=`echo $1 | sed 's/.gq$//'`
    basedir="`echo $1 | sed 's/\/[^\/]*$//'`/."

    echo -n "$basename..."

    echo 1>&2
    echo "###### Testing $basename" 1>&2

    generatedfiles=""

    #generatedfiles="$generatedfiles ${basename}.i.out" &&
    #Run "$graphquil" "-i" "<" $1 ">" ${basename}.i.out &&
    #Compare ${basename}.i.out ${reffile}.out ${basename}.i.diff

    generatedfiles="$generatedfiles ${basename}.j.out" &&
    Run "$graphquil" "-c" $1 ">" ${basename}.j.out &&
    Compare ${basename}.j.out ${reffile}.out ${basename}.j.diff

    # Report the status and clean up the generated files

    if [ $error -eq 0 ] ; then
	if [ $keep -eq 0 ] ; then
	    rm -f $generatedfiles
	fi
	echo "OK"
	echo "###### SUCCESS" 1>&2
    else
	echo "###### FAILED" 1>&2
	globalerror=$error
    fi
}

TestRunningProgram() {
    error=0
    basename=`echo $1 | sed 's/.*\\///
                             s/.gq//'`
    reffile=`echo $1 | sed 's/.gq$//'`
    basedir="`echo $1 | sed 's/\/[^\/]*$//'`/."

    # echo -n "$basename..."
    echo -n "$basename..."

    echo 1>&2
    echo "###### Testing $basename" 1>&2

    generatedfiles=""
    tmpfiles=""

    # old from microc - interpreter
    # generatedfiles="$generatedfiles ${basename}.i.out" &&
    # Run "$lorax" "-i" "<" $1 ">" ${basename}.i.out &&
    # Compare ${basename}.i.out ${reffile}.out ${basename}.i.diff

    generatedfiles="$generatedfiles ${basename}.j.out" &&
    tmpfiles="$tmpfiles " &&
    Run "$graphquil" "-j" $1 &&
    Run "$java_output" ">" ${basename}.j.out &&
    Compare ${basename}.j.out ${reffile}.out ${basename}.f.diff
    
    rm -f $tmpfiles

    # Report the status and clean up the generated files

    if [ $error -eq 0 ] ; then
    if [ $keep -eq 0 ] ; then
        rm -f $generatedfiles
    fi
    echo "OK"
    echo "###### SUCCESS" 1>&2
    else
    echo "###### FAILED" 1>&2
    globalerror=$error
    fi
}

while getopts kdpsh c; do
    case $c in
	k) # Keep intermediate files
	    keep=1
	    ;;
	h) # Help
	    Usage
	    ;;
    esac
done

shift `expr $OPTIND - 1`

if [ $# -ge 1 ]
then
    files=$@
else
    files="tests/test-*.gq"
fi

for file in $files
do
    case $file in
    *test-parser*)
        CheckParser $file 2>> $globallog
        ;;
    *test-sc*)
        CheckSemanticAnalysis $file 2>> $globallog
        ;;
    *test-full*)
        TestRunningProgram $file 2>> $globallog
        ;;
    *test-fail*)
        CheckFail $file 2>> $globallog
        ;;
	*test-*)
	    Check $file 2>> $globallog
	    ;;
	*)
	    echo "unknown file type $file"
	    globalerror=1
	    ;;
    esac
done

exit $globalerror
